#include <htypes.h>
#include <arch/system.h>
#include <arch/pinmux.h>

#define read8(addr) \
	({ unsigned char __v = (*(volatile unsigned char *) (addr)); __v; })
#define read16(addr) \
	({ unsigned short __v = (*(volatile unsigned short *) (addr)); __v; })
#define read32(addr) \
	({ unsigned int __v = (*(volatile unsigned int *) (addr)); __v; })

#define write8(addr, b) \
	(void)((*(volatile unsigned char *) (addr)) = (b))
#define write16(addr, b) \
	(void)((*(volatile unsigned short *) (addr)) = (b))
#define write32(addr, b) \
	(void)((*(volatile unsigned int *) (addr)) = (b))

/* CPG */
#define FRQCRA		0xe6150000
#define FRQCRB		0xe6150004
#define FRQCRC		0xe61500e0
#define FRQCRD		0xe61500e4
#define SUBCKCR		0xe6150080
#define PLLC01CR	0xe6150028
#define PLLC2CR		0xe615002c
#define RMSTPCR4	0xe6150120
#define SMSTPCR1	0xe6150134
#define SMSTPCR4	0xe6150140

/* BSC */
#define CMNCR		0xfec10000
#define CS0BCR		0xfec10004
#define CS2BCR		0xfec10008
#define CS4BCR		0xfec10010
#define CS5ABCR		0xfec10014
#define CS5BBCR		0xfec10018
#define CS6ABCR		0xfec1001c
#define CS0WCR		0xfec10024
#define CS2WCR		0xfec10028
#define CS4WCR		0xfec10030
#define CS5AWCR		0xfec10034
#define CS5BWCR		0xfec10038
#define CS6AWCR		0xfec1003c
#define RBWTCNT		0xfec10054
#define CS0WCR2		0xfec10224
#define CS2WCR2		0xfec10228
#define CS4WCR2		0xfec10230

/* DBSC */
#define DBACEN		0xfe400010
#define DBRFEN		0xfe400014
#define DBCMD		0xfe400018
#define DBWAIT		0xfe40001c
#define DBKIND		0xfe400020
#define DBCONF0		0xfe400024
#define DBPHYTYPE	0xfe400030
#define DBBL		0xfe4000b0
#define DBTR0		0xfe400040
#define DBTR1		0xfe400044
#define DBTR2		0xfe400048
#define DBTR3		0xfe400050
#define DBTR4		0xfe400054
#define DBTR5		0xfe400058
#define DBTR6		0xfe40005c
#define DBTR7		0xfe400060
#define DBTR8		0xfe400064
#define DBTR9		0xfe400068
#define DBTR10		0xfe40006c
#define DBTR11		0xfe400070
#define DBTR12		0xfe400074
#define DBTR13		0xfe400078
#define DBTR14		0xfe40007c
#define DBTR15		0xfe400080
#define DBTR16		0xfe400084
#define DBTR17		0xfe400088
#define DBTR18		0xfe40008c
#define DBTR19		0xfe400090
#define DBRFCNF0	0xfe4000e0
#define DBRFCNF1	0xfe4000e4
#define DBRFCNF2	0xfe4000e8
#define DBRFCNF3	0xfe4000ec
#define DBRNK0		0xfe400100
#define DBDFICNT	0xfe400244
#define FUNCCTRL	0xc12a0000
#define DLLCTRL		0xc12a0004
#define ZQCALCTRL	0xc12a0008
#define ZQODTCTRL	0xc12a000c
#define RDCTRL		0xc12a0010
#define RDTMG		0xc12a0014
#define FIFOINIT	0xc12a0018
#define OUTCTRL		0xc12a001c
#define DQCALOFS1	0xc12a00e8
#define DQCALOFS2	0xc12a00ec
#define DQCALEXP	0xc12a00f8
#define DDRPNCNT	0xe605803c

/* TMU */
#define TSTR		0xfff80004
#define TCOR0		0xfff80008
#define TCNT0		0xfff8000c
#define TCR0		0xfff80010

#if defined(DEBUG)
#define __noinline	__attribute__((__noinline__))
#else
#define __noinline
#endif
#define __maybe_unused	__attribute__((__unused__))
#define __section(S)	__attribute__((__section__(#S)))
#define __init		__section(".pre.init") __noinline
#define __initdata	__section(".pre.init.data")

#define A800xBSPIF0		PIN_FMSOOLR
#define A800xBSPIF1		PIN_FMSOOBT
#define A800xBSPIF2		PIN_CS4_N
#define AEVAxLED1		PIN_FRB
#define AEVAxLED2		PIN_CS4_N
#define AEVAxLED3		PIN_CS2_N
#define AEVAxLED4		PIN_WAIT_N

/* bspif */
#define BSPIF_I2C0_EEPROM_A0	0
#define BSPIF_CRYPTOMEMORY	1

/* board_type */
#define BOARD_ARMADILLO800EVA	0
#define BOARD_ARMADILLO810	1

struct board_info {
	unsigned int board_id;
	unsigned int dram_id;
};

static struct board_info __initdata board_info = { 0, 0 };

static void __init arch_gpio_output(unsigned int pin, unsigned char level)
{
	if (level)
		write32(PIN_GPIODSR(pin), 1 << PIN_GPIO32(pin));
	else
		write32(PIN_GPIODCR(pin), 1 << PIN_GPIO32(pin));
}

static unsigned char __init arch_gpio_input(unsigned int pin)
{
	unsigned int val;
	val = read32(PIN_GPIODCR(pin)) & (1 << PIN_GPIO32(pin));
	return val ? 1 : 0;
}

static void __init arch_udelay(unsigned int usecs)
{
	unsigned int count;

	/* SUB = 50MHz, TCNT_CLK = 12.5MHz */
	count = (12 * usecs) + (usecs / 2) + (usecs & 1);

	write16(TCR0, 0);
	write32(TCOR0, count);
	write32(TCNT0, count);

	write8(TSTR, 1);
	while (!(read16(TCR0) & 0x100));
	write16(TCR0, 0x0000);
	write8(TSTR, 0);
}

#if defined(DEBUG)
static void __init __maybe_unused arch_print_init(void)
{
	pinmux(PIN_SCIFA_TXD_1, MUX(1, 1, 0, PULL_OFF));
	pinmux(PIN_SCIFA_RXD_1, MUX(1, 0, 1, PULL_DOWN));

	write16(0xE6C50008, 0x0030);
	write16(0xE6C50008, 0x0030);
	write16(0xE6C50000, 0);
	write16(0xE6C50000, 0);
	write16(0xE6C50018, 0x0006);

	write16(0xE6C50018, 0);
	write8(0xE6C50004, 0x1a);
}

static void __init __maybe_unused arch_print(const char *buf)
{
	char *ptr = (char *)buf;
	while (*ptr) {
		while (!((read16(0xe6c5001c) >> 8) < 0x40));
		write8(0xE6C50020, *(ptr++));
	}
}
#else
#define arch_print_init()
#define arch_print(buf)
#endif

static void __init arch_dram_init(unsigned int conf)
{
	/* DBSC */
	write32(DBCMD, 0x20000000);
	write32(DBCMD, 0x10009C40);
	arch_udelay(50);
	write32(DBKIND, 0x00000007);
	write32(DBCONF0, 0x0E030A02);
	write32(DBPHYTYPE, 0x00000001);
	write32(DBBL, 0x00000000);
	write32(DBTR0, 0x00000006);
	write32(DBTR1, 0x00000005);
	write32(DBTR2, 0x00000000);
	write32(DBTR3, 0x00000006);
	write32(DBTR4, 0x00080006);
	write32(DBTR5, 0x00000015);
	write32(DBTR6, 0x0000000F);
	write32(DBTR7, 0x00000004);
	write32(DBTR8, 0x00000018);
	write32(DBTR9, 0x00000006);
	write32(DBTR10, 0x00000006);
	write32(DBTR11, 0x0000000F);
	write32(DBTR12, 0x0000000D);
	write32(DBTR13, 0x000000A0);
	write32(DBTR14, 0x000A0003);
	write32(DBTR15, 0x00000003);
	write32(DBTR16, 0x40005005);
	write32(DBTR17, 0x0C0C0000);
	write32(DBTR18, 0x00000200);
	write32(DBTR19, 0x00000040);
	write32(DBRNK0, 0x00000001);
	write32(DBDFICNT, 0x00000110);
	write32(FUNCCTRL, 0x00000101);
	write32(DLLCTRL, 0x00000001);
	write32(ZQCALCTRL, 0x00000186);
	write32(ZQODTCTRL, 0xB3440051);
	write32(RDCTRL, 0x94449443);
	write32(RDTMG, 0x000000C0);
	write32(FIFOINIT, 0x00000101);
	write32(OUTCTRL, 0x02060506);
	write32(DQCALOFS1, 0x00004646);
	write32(DQCALOFS2, 0x00004646);
	write32(DQCALEXP, 0x800000AA);
	write32(DLLCTRL, 0x00000000);
	write32(DDRPNCNT, 0x00000000);

	write32(DBCMD, 0x0000000C);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DDRPNCNT, 0x00000002);

	write32(DBCMD, 0x0000000C);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(ZQCALCTRL, 0x00000187);
	write32(DBCMD, 0x00009C40);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x00009C40);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBDFICNT, 0x00000010);
	write32(OUTCTRL, 0x02060507);

	write32(DBCMD, 0x00009C40);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x21009C40);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x00009C40);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x00009C40);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x00009C40);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x00009C40);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x11000044);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x2A000000);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x2B000000);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x29000004);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x28001520);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBCMD, 0x03000200);
	write32(DBWAIT, 0xFE400020);
	arch_udelay(50);

	write32(DBRFCNF0, 0x000001FF);
	write32(DBRFCNF1, 0x00010C30);
	write32(DBRFCNF2, 0x00000000);
	write32(DBRFEN, 0x00000001);
	write32(DBACEN, 0x00000001);
}

static void __init armadillo800eva_early_configure(void)
{
	/* LED1.ON, LED2/3/4.OFF */
	arch_gpio_output(AEVAxLED1, 1);
	arch_gpio_output(AEVAxLED2, 0);
	arch_gpio_output(AEVAxLED3, 0);
	arch_gpio_output(AEVAxLED4, 0);
	pinmux(AEVAxLED1, MUX(0, 1, 0, PULL_OFF));
	pinmux(AEVAxLED2, MUX(0, 1, 0, PULL_OFF));
	pinmux(AEVAxLED3, MUX(0, 1, 0, PULL_OFF));
	pinmux(AEVAxLED4, MUX(0, 1, 0, PULL_OFF));
}

static void __init arch_read_board_info(void)
{
	unsigned char bspif = 0;

	/* get parameter device */
	pinmux(A800xBSPIF0, MUX(0, 0, 1, PULL_DOWN));
	pinmux(A800xBSPIF1, MUX(0, 0, 1, PULL_DOWN));
	pinmux(A800xBSPIF2, MUX(0, 0, 1, PULL_DOWN));
	bspif |= (arch_gpio_input(A800xBSPIF0) << 0);
	bspif |= (arch_gpio_input(A800xBSPIF1) << 1);
	bspif |= (arch_gpio_input(A800xBSPIF2) << 2);

	/* temporary implementation of board detection
	 *   actually, read information from devices (EEPROM or CryptoMemory).
	 *   but, we omit detailed implementation now.
	 */
	if (bspif == BSPIF_I2C0_EEPROM_A0) {
		/* Armadillo-800 EVA */
		board_info.board_id = 0x08000000;
		board_info.dram_id = 0x00001d05; /* DDR3, 512MB, 32bit */
	} else {
		/* Armadillo-810 */
		board_info.board_id = 0x08100000;
		board_info.dram_id = 0x00001d05; /* DDR3, 512MB, 32bit */
	}
}

void __init arch_init(void)
{
	/* RWT Setting */
	write16(0xE6020004, 0xa500);
	write16(0xE6030004, 0xa500);

	/* CPG */
	write32(RMSTPCR4, 0xff800080);
	write32(SMSTPCR4, 0xff800080);
	write32(SMSTPCR1, 0xfdffdfff);

	write32(FRQCRB, 0x00000000);
	write32(FRQCRA, 0x62030533);
	write32(FRQCRC, 0x208a354e);
	write32(FRQCRB, 0x80331050);
	while (read32(FRQCRB) & 0x80000000); /* waiting for setting complete */

	write32(FRQCRD, 0x00000000);
	write32(SUBCKCR, 0x0000010B);
	write32(PLLC01CR, 0x00004000);
	write32(PLLC2CR, 0xa0000000);
	while (!(read32(PLLC2CR) & 0x80000000));

	arch_print_init();

	arch_read_board_info();

	if (board_info.board_id == 0x08000000)
		/* Armadillo-800 EVA */
		armadillo800eva_early_configure();

	/* BSC */
	write32(CMNCR, 0x0000001B);
	write32(CS0BCR, 0x00410400);
	write32(CS2BCR, 0x00410400);
	write32(CS5BBCR, 0x00410400);
	write32(CS6ABCR, 0x02CB0400);
	write32(CS0WCR, 0x00000440);
	write32(CS2WCR, 0x00000440);
	write32(CS5BWCR, 0x00000240);
	write32(CS6AWCR, 0x00000240);
	write32(RBWTCNT, 0x00000005);
	write32(CS0WCR2, 0x00000002);
	write32(CS2WCR2, 0x00000002);
	write32(CS4WCR2, 0x00000002);

	arch_dram_init(board_info.dram_id);
}
